{
  Example how to create the new 97menus
  ->MenuEx97Utils.pas
  ->Utility functions
  Author : Marcel Ohlmeier
  e-mail : ohlmeier@swhamm.de           
  This example is freeware
  feel free to use this in your programs
  but credit me :-) 
}
unit MenuEx97Utils;

interface

uses  WinTypes, Windows, SysUtils, Graphics, menus;

procedure DrawDisabledText(theCanvas: TCanvas; TempRect: TRect; TempCaption: PChar);
procedure DrawDisabledBitmap(Canvas: TCanvas; x, y: Integer; bmp: TBitmap);
procedure DrawTransparentBitmap(canvas: TCanvas; bmp: TBitmap; xStart,yStart: Integer; cTransparentColor: LongInt);
procedure SetMenuItemCheckedState(Menu : TMainMenu; Item : Integer; YesNo : Boolean);
procedure SetMenuItemDisabledState(Menu : TMainMenu; Item : Integer;YesNo : Boolean);
function  IsMenuItemChecked(Menu : Integer; Item : Integer) : Boolean;
function  IsMenuItemDisabled(Menu : Integer; Item : Integer) : Boolean;

implementation

//function to check if a menuitem is disabled
function IsMenuItemDisabled(Menu : Integer; Item : Integer) : Boolean;
var
   state : word;
begin
   State := GetMenuState(Menu, Item, MF_BYCOMMAND);
   If State AND MF_DISABLED <>0 then
      result := True
   else
      result := False;
end;

//function to check if a menuItem is checked
function IsMenuItemChecked(Menu : Integer; Item : Integer) : Boolean;
var
   state : Word;
begin
   State := GetMenuState(Menu, Item, MF_BYCOMMAND);
   If State AND MF_CHECKED <>0 then
      result := True
   else
      result := False;
end;

///procedure to display a disabled menuitem
procedure SetMenuItemDisabledState(Menu : TMainMenu; Item : Integer;YesNo : Boolean);
begin
   If YesNo = True then
      ModifyMenu(Menu.Handle,Item,MF_BYCOMMAND+MF_OWNERDRAW+MF_DISABLED,Item, nil)
   else
      ModifyMenu(Menu.Handle,Item,MF_BYCOMMAND+MF_OWNERDRAW,Item, nil)

end;

//procedure to display a checked or unchecked menuitem
procedure SetMenuItemCheckedState(Menu : TMainMenu; Item : Integer; YesNo : Boolean);
begin
   If YesNo = True then
      ModifyMenu(Menu.Handle,Item,MF_BYCOMMAND+MF_OWNERDRAW+MF_CHECKED,Item, nil)
   else
      ModifyMenu(Menu.Handle,Item,MF_BYCOMMAND+MF_OWNERDRAW,Item, nil)
end;

{DrawDisabledText from
Fabrice Deville  fdev@tornado.be - TExplorerButton }
procedure DrawDisabledText(theCanvas: TCanvas; TempRect: TRect; TempCaption: PChar);
begin
	theCanvas.Brush.Style := bsClear;
	theCanvas.Font.Color := clBtnHighlight;
	with TempRect do
	begin
		left := left + 1;
		top  := top + 1;
		right:= right + 1;
		bottom:= bottom + 1;
	end;
	DrawText(theCanvas.handle, PChar(TempCaption), StrLen(PChar(TempCaption)), TempRect, DT_LEFT);
	theCanvas.Font.Color := clBtnShadow;
	with TempRect do
	begin
		left := left - 1;
		top  := top - 1;
		right:= right - 1;
		bottom:= bottom - 1;
	end;
	DrawText(theCanvas.handle, PChar(TempCaption), StrLen(PChar(TempCaption)), TempRect, DT_LEFT);
end;

{DrawDisabledImage from
Fabrice Deville  fdev@tornado.be - TExplorerButton }
procedure DrawDisabledBitmap(Canvas: TCanvas; x, y: Integer; bmp: TBitmap);
var MonoBmp: TBitmap;
begin
   MonoBmp := TBitmap.Create;
   try
      MonoBmp.Assign(bmp);
      MonoBmp.Canvas.Brush.Color := clBlack;
      MonoBmp.Monochrome := True;
      Canvas.Brush.Color := clBtnHighlight;
      SetTextColor(Canvas.Handle, clBlack);
      SetBkColor(Canvas.Handle, clWhite);
      BitBlt(Canvas.Handle, x+1, y+1, bmp.Width, bmp.Height,
      MonoBmp.Canvas.Handle, 0, 0, $00E20746);
      Canvas.Brush.Color := clBtnShadow;
      SetTextColor(Canvas.Handle, clBlack);
      SetBkColor(Canvas.Handle, clWhite);
      BitBlt(Canvas.Handle, x, y, bmp.Width, bmp.Height,
      MonoBmp.Canvas.Handle, 0, 0, $00E20746);
   finally
      MonoBmp.Free;
   end
end;

{DrawTransparentBitmap from
Fabrice Deville  fdev@tornado.be - TExplorerButton }
procedure DrawTransparentBitmap(canvas: TCanvas; bmp: TBitmap; xStart,yStart: Integer; cTransparentColor: LongInt);
var bm: WinTypes.TBitmap;
   cColor: TColorRef;
   bmAndBack, bmAndObject, bmAndMem, bmSave, oldBmp: HBITMAP;
   bmBackOld, bmObjectOld, bmMemOld, bmSaveOld, hBmp: HBITMAP;
   hdcMem, hdcBack, hdcObject, hdcTemp, hdcSave, dc: HDC;
   ptSize: WinTypes.TPoint;
   temp_bitmap: TBitmap;
begin
   temp_bitmap := TBitmap.Create;
   temp_bitmap.Assign(bmp);
   try
      dc := canvas.Handle;
      hBmp := temp_bitmap.Handle;
      hdcTemp := CreateCompatibleDC(dc);
      oldBmp := SelectObject(hdcTemp, hBmp);

      GetObject(hBmp, SizeOf(bm), @bm);
      ptSize.x := bm.bmWidth;
      ptSize.y := bm.bmHeight;

      hdcBack   := CreateCompatibleDC(dc);
      hdcObject := CreateCompatibleDC(dc);
      hdcMem    := CreateCompatibleDC(dc);
      hdcSave   := CreateCompatibleDC(dc);

      bmAndBack   := CreateBitmap(ptSize.x, ptSize.y, 1, 1, nil);

      bmAndObject := CreateBitmap(ptSize.x, ptSize.y, 1, 1, nil);

      bmAndMem    := CreateCompatibleBitmap(dc, ptSize.x, ptSize.y);
      bmSave      := CreateCompatibleBitmap(dc, ptSize.x, ptSize.y);

      bmBackOld   := SelectObject(hdcBack, bmAndBack);
      bmObjectOld := SelectObject(hdcObject, bmAndObject);
      bmMemOld    := SelectObject(hdcMem, bmAndMem);
      bmSaveOld   := SelectObject(hdcSave, bmSave);

      SetMapMode(hdcTemp, GetMapMode(dc));

      BitBlt(hdcSave, 0, 0, ptSize.x, ptSize.y, hdcTemp, 0, 0, SRCCOPY);

      cColor := SetBkColor(hdcTemp, cTransparentColor);

      BitBlt(hdcObject, 0, 0, ptSize.x, ptSize.y, hdcTemp, 0, 0, SRCCOPY);

      SetBkColor(hdcTemp, cColor);

      BitBlt(hdcBack, 0, 0, ptSize.x, ptSize.y, hdcObject, 0, 0, NOTSRCCOPY);
      BitBlt(hdcMem, 0, 0, ptSize.x, ptSize.y, dc, xStart, yStart, SRCCOPY);
      BitBlt(hdcMem, 0, 0, ptSize.x, ptSize.y, hdcObject, 0, 0, SRCAND);
      BitBlt(hdcTemp, 0, 0, ptSize.x, ptSize.y, hdcBack, 0, 0, SRCAND);
      BitBlt(hdcMem, 0, 0, ptSize.x, ptSize.y, hdcTemp, 0, 0, SRCPAINT);
      BitBlt(dc, xStart, yStart, ptSize.x, ptSize.y, hdcMem, 0, 0, SRCCOPY);
      BitBlt(hdcTemp, 0, 0, ptSize.x, ptSize.y, hdcSave, 0, 0, SRCCOPY);

      DeleteObject(SelectObject(hdcBack, bmBackOld));
      DeleteObject(SelectObject(hdcObject, bmObjectOld));
      DeleteObject(SelectObject(hdcMem, bmMemOld));
      DeleteObject(SelectObject(hdcSave, bmSaveOld));

      SelectObject(hdcTemp, oldBmp);

      DeleteDC(hdcMem);
      DeleteDC(hdcBack);
      DeleteDC(hdcObject);
      DeleteDC(hdcSave);
      DeleteDC(hdcTemp);
   finally
      temp_bitmap.Free;
   end;
end;
end.
 